<?php

namespace App\Http\Controllers\Admin;

use App\Models\Card;
use App\Models\User;
use Inertia\Inertia;
use App\Models\Gateway;
use App\Models\CardOrder;
use App\Models\CardHolder;
use App\Models\CreditCard;
use App\Models\Notification;
use Illuminate\Http\Request;
use App\Services\StripeHelper;
use Illuminate\Support\Facades\DB;
use App\Http\Controllers\Controller;

class CardOrderController extends Controller
{

    public function __construct()
    {
        $this->middleware('permission:card-orders');
    }
    
    public function index(Request $request)
    {
        $segments = request()->segments();
        $buttons = [
            [
                'name' => '<i class="bx bx-file"></i>&nbsp&nbsp' . __('Cardholder Data'),
                'url' => '#',
                'target' => '#cardholderDataDrawer',
            ],
            [
                'name' => '<i class="bx bx-cart"></i>&nbsp&nbsp' . __('Manual Order'),
                'url' => '#',
                'target' => '#manualOrderDrawer',
            ],

        ];
        $orders = CardOrder::query()
            ->with(['user', 'card', 'gateway', 'credit_card:id,card_order_id,card'])
            ->filterOn(['invoice_no', 'status'])
            ->filterOnRelation(['card_title', 'user_email'])
            ->latest()->paginate(20);

        $totalOrders = CardOrder::count();
        $totalPendingOrders = CardOrder::where('status', 'pending')->count();
        $totalApprovedOrders = CardOrder::where('status', 'approved')->count();
        $totalRejectedOrders = CardOrder::where('status', 'rejected')->count();
        $type = $request->type ?? 'email';
        $cardholderData = get_option('cardholder_data');

        // for orders
        $cards = Card::query()
            ->where('status', 'active')
            ->with('category:id,title,icon')
            ->get();
        $gateways = Gateway::query()->where('status', 1)->get();
        if ($request->filled('user')) {
            $findUser = User::query()->where(function ($query) {
                $query->where('email', request()->user)
                    ->orWhere('phone', request()->user);
            })->first();
        }
        return Inertia::render('Admin/CardOrder/Index', [
            'segments' => $segments,
            'orders' => $orders,
            'request' => $request,
            'totalOrders' => $totalOrders,
            'totalPendingOrders' => $totalPendingOrders,
            'totalApprovedOrders' => $totalApprovedOrders,
            'totalRejectedOrders' => $totalRejectedOrders,
            'type' => $type,
            'buttons' => $buttons,
            'cardholderData' => $cardholderData,
            'cards' => $cards,
            'gateways' => $gateways,
            'findUser' => $findUser ?? null,
        ]);
    }

    public function show($id)
    {
        $order = CardOrder::with([
            'user' => [
                'country:id,name',
                'state:id,name',
                'city:id,name',
            ],
            'card',
            'gateway'
        ])->findOrFail($id);
        $creditCard = CreditCard::with([
            'user' => [
                'country',
                'state',
                'city',
            ],
            'card:id,card_variant',
            'cardHolder'
        ])
            ->where('card_order_id', $order->id)->first();

        $invoice_data = get_option('invoice_data');

        return Inertia::render('Admin/CardOrder/Show', [
            'order' => $order,
            'invoice_data' => $invoice_data,
            'meta' => $order->meta,
            'creditCard' => $creditCard ?? null,
        ]);
    }

    public function update(Request $request, StripeHelper $stripeHelper, $id)
    {
        $order = CardOrder::with([
            'user' => [
                'country',
                'state',
                'city',
            ],
            'card'
        ])->findOrFail($id);

        DB::transaction(function () use ($stripeHelper, $order, $request) {
            $order->update([
                'status' => $request->status,
            ]);
            $isUserHasCardholder = CardHolder::where('user_id', $order->user->id);
            if ($request->status == 'approved') {
                if ($isUserHasCardholder->exists()) {
                    $cardholder = $isUserHasCardholder->first();
                    if ($cardholder->status == 'active') {
                        $stripeHelper->storeCreditCard($cardholder, $order, true);
                        $order->update(['status' => 'active']);
                    }
                } else {
                    $cardholder = $stripeHelper->storeCardHolder($order->user);
                    if ($cardholder->status == 'active') {
                        $stripeHelper->storeCreditCard($cardholder, $order);
                        $order->update(['status' => 'active']);
                    }
                }
            }
        });

        return back();
    }

    public function store(Request $request)
    {
        $request->validate([
            'card_id' => 'required',
            'payment_id' => 'required',
            'user_id' => 'required',
            'gateway_id' => 'required',
            'amount' => 'required',
        ]);
        DB::transaction(function () use ($request) {
            $tax = get_option('tax');
            $order = CardOrder::create([
                'status' => 'pending',
                'card_id' => $request->card_id,
                'payment_id' => $request->payment_id,
                'user_id' => $request->user_id,
                'gateway_id' => $request->gateway_id,
                'amount' => $request->amount,
                'tax' => $tax,
            ]);
            Notification::create([
                'user_id' => auth()->id(),
                'title' => __('New Card Order submitted'),
                'comment' => __('A  Card Order has been submitted'),
                'url' => route('user.card-orders.show', $order->id),
                'for_admin' => 0,
            ]);
        });

        return back()->with('success', __('Card Order Created Successfully'));
    }
    public function updateStatus(Request $request, $id)
    {
        $order = CardOrder::findOrFail($id);
        $order->update([
            'status' => $request->status,
        ]);

        return back();
    }
}
