<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\CreditCard;
use App\Services\StripeHelper;
use Illuminate\Http\Request;
use Inertia\Inertia;
use App\Models\Notification;

class CreditCardController extends Controller
{

    public function __construct()
    {
        $this->middleware('permission:credit-card');
    }
    public function index(Request $request)
    {
        $creditCards = CreditCard::query()
            ->filterOn(['title', 'last4', 'status'])
            ->with(['user', 'card_order'])
            ->latest()
            ->paginate(12);

        $totalCards = CreditCard::count();
        $totalPendingCards = CreditCard::where('status', 'pending')->count();
        $totalApprovedCards = CreditCard::where('status', 'approved')->count();
        $totalRejectedCards = CreditCard::where('status', 'rejected')->count();
        return Inertia::render('Admin/CreditCard/Index', [
            'creditCards' => $creditCards,
            'totalCards' => $totalCards,
            'totalPendingCards' => $totalPendingCards,
            'totalApprovedCards' => $totalApprovedCards,
            'totalRejectedCards' => $totalRejectedCards,
            'request' => $request,
        ]);
    }

    public function show($id, StripeHelper $stripeHelper)
    {
        $creditCard = CreditCard::with([
            'user' => [
                'country',
                'state',
                'city',
            ],
            'card_order',
            'virtual_card:id,card_variant'
        ])->findOrFail($id);
        $cardholder_data = get_option('cardholder_data');

        $stripeData = $stripeHelper->transactions('user.credit-card.transactions', null, $creditCard->card);
        $stripeCurrency = env('STRIPE_CURRENCY', 'usd');
        return Inertia::render('Admin/CreditCard/Show', [
            'creditCard' => $creditCard,
            'cardholder_data' => $cardholder_data,
            'transactions' => $stripeData['transactions'] ?? [],
            'has_more' => $stripeData['has_more'] ?? false,
            'next' => $stripeData['next'] ?? '',
            'prev' => $stripeData['prev'] ?? '',
            'stripe_public_key' => env('STRIPE_PUBLIC_API_KEY'),
            'stripe_currency' => $stripeCurrency
        ]);
    }


    public function transactions(StripeHelper $stripeHelper)
    {
        $stripeData = $stripeHelper->transactions('admin.credit-card.transactions');
        $stripeCurrency = env('STRIPE_CURRENCY', 'usd');
        return Inertia::render('Admin/Transaction/Index', [
            'transactions' => $stripeData['transactions'] ?? [],
            'has_more' => $stripeData['has_more'] ?? false,
            'next' => $stripeData['next'] ?? '',
            'prev' => $stripeData['prev'] ?? '',
            'stripe_currency' => $stripeCurrency
        ]);
    }


    public function authorizations(StripeHelper $stripeHelper)
    {
        $stripeData = $stripeHelper->authorizations('admin.credit-card.authorizations');
        $stripeCurrency = env('STRIPE_CURRENCY', 'usd');
        return Inertia::render('Admin/Authorization/Index', [
            'authorizations' => $stripeData['authorizations'] ?? [],
            'has_more' => $stripeData['has_more'] ?? [],
            'next' => $stripeData['next'] ?? '',
            'prev' => $stripeData['prev'] ?? '',
            'stripe_currency' => $stripeCurrency
        ]);
    }

    public function updateStatus($id, StripeHelper $stripeHelper)
    {
        $creditCard = CreditCard::findOrFail($id);
        $status = match ($creditCard->status) {
            'active' => 'inactive',
            'inactive' => 'active',
        };
        $creditCard->update([
            'status' => $status,
        ]);
        $stripeHelper->updateCardStatus($creditCard->card, $status);
        Notification::create([
            'user_id' => auth()->id(),
            'title' => __('Credit Card Status Updated'),
            'comment' => __('Credit Card status updated to :status', ['status' => $status]),
            'url' => route('user.credit-cards.show', $creditCard->id),
            'for_admin' => 0,
        ]);

        return back()->with('success', 'Updated Successfully');
    }

    public function updateSpending($id, StripeHelper $stripeHelper)
    {
        $creditCard = CreditCard::findOrFail($id);
        $card = $stripeHelper->updateSpendingLimits($creditCard->card, request('amount'));
        $creditCard->update([
            'spending_limits' => $card['spending_limits'] / 100,
        ]);
        Notification::create([
            'user_id' => auth()->id(),
            'title' => __('Credit Card Spending Limits Updated'),
            'comment' => __('Credit Card spending limits updated to'),
            'url' => route('user.credit-cards.show', $creditCard->id),
            'for_admin' => 0,
        ]);
        return back()->with('warning', 'Updated Successfully');
    }
}
