<?php

namespace App\Http\Controllers\User;

use App\Http\Controllers\Controller;
use App\Models\CardHolder;
use App\Models\CreditCard;
use App\Models\Gateway;
use App\Services\StripeHelper;
use Illuminate\Http\Request;
use Inertia\Inertia;

class CreditCardController extends Controller
{
    public function index(Request $request)
    {
        $creditCards = CreditCard::query()
            ->filterOn(['title', 'last4', 'status'])
            ->where('user_id', auth()->id())
            ->with(['user', 'card:id,card_variant'])
            ->paginate(12);

        $totalCards = CreditCard::count();
        $totalPendingCards = CreditCard::where('status', 'pending')->count();
        $totalApprovedCards = CreditCard::where('status', 'approved')->count();
        $totalRejectedCards = CreditCard::where('status', 'rejected')->count();
        return Inertia::render('User/CreditCard/Index', [
            'creditCards' => $creditCards,
            'totalCards' => $totalCards,
            'totalPendingCards' => $totalPendingCards,
            'totalApprovedCards' => $totalApprovedCards,
            'totalRejectedCards' => $totalRejectedCards,
            'request' => $request,
        ]);
    }

    public function show($id, StripeHelper $stripeHelper)
    {
        $buttons = [
            [
                'name' => '<i class="bx bx-plus"></i>&nbsp&nbsp' . __('Top up account'),
                'url' => route('user.top-up.create'),
            ]
        ];
        $creditCard = CreditCard::with(['user' => [
            'country',
            'state',
            'city',
        ], 'card_order', 'virtual_card:id,card_variant'])->findOrFail($id);
        $cardholder_data = get_option('cardholder_data');
        $totalSpend = $stripeHelper->currentSpend($creditCard->card);
        $gateways = Gateway::query()->where('status', 1)->get();

        $stripeData = $stripeHelper->transactions('user.credit-card.transactions', null, $creditCard->card);
        $stripeCurrency = env('STRIPE_CURRENCY', 'usd');
        return Inertia::render('User/CreditCard/Show', [
            'creditCard' => $creditCard,
            'cardholder_data' => $cardholder_data,
            'totalSpend' => $totalSpend,
            'buttons' => $buttons,
            'gateways' => $gateways,
            'transactions' => $stripeData['transactions'],
            'has_more' => $stripeData['has_more'],
            'next' => $stripeData['next'],
            'prev' => $stripeData['prev'],
            'stripe_public_key' => env('STRIPE_PUBLIC_API_KEY'),
            'stripe_currency' => $stripeCurrency
        ]);
    }


    public function transactions(StripeHelper $stripeHelper)
    {
        $cardholder = CardHolder::query()->where('user_id', auth()->id());
        if (!$cardholder->exists()) {
            return Inertia::render('User/Transaction/Index', [
                'transactions' => ['data' => []],

            ]);
        }
        $stripeData = $stripeHelper->transactions('user.credit-card.transactions', $cardholder->first()->cardholder);
        $stripeCurrency = env('STRIPE_CURRENCY', 'usd');
        return Inertia::render('User/Transaction/Index', [
            'transactions' => $stripeData['transactions'],
            'has_more' => $stripeData['has_more'],
            'next' => $stripeData['next'],
            'prev' => $stripeData['prev'],
            'stripe_currency' => $stripeCurrency
        ]);
    }


    public function authorizations(StripeHelper $stripeHelper)
    {
        $cardholder = CardHolder::query()->where('user_id', auth()->id());
        if (!$cardholder->exists()) {
            return Inertia::render('User/Authorization/Index', [
                'transactions' => ['data' => []],
            ]);
        }

        $stripeData = $stripeHelper->authorizations('user.credit-card.authorizations', $cardholder->first()->cardholder);
        $stripeCurrency = env('STRIPE_CURRENCY', 'usd');
        return Inertia::render('User/Authorization/Index', [
            'authorizations' => $stripeData['authorizations'],
            'has_more' => $stripeData['has_more'],
            'next' => $stripeData['next'],
            'prev' => $stripeData['prev'],
            'stripe_currency' => $stripeCurrency
        ]);
    }

    public function updateStatus($id, StripeHelper $stripeHelper)
    {
        $creditCard = CreditCard::findOrFail($id);
        $stripeHelper->updateCardStatus($creditCard->card);
        $creditCard->update([
            'status' => 'inactive',
        ]);

        return back()->with('warning', 'Updated Successfully');
    }
}
