<?php

namespace App\Http\Controllers\User;

use Carbon\Carbon;
use App\Models\Plan;
use Inertia\Inertia;
use App\Models\Order;
use App\Helpers\PageHeader;
use App\Helpers\Toastr;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\CardHolder;
use App\Models\CardOrder;
use App\Models\CreditCard;
use App\Models\TopUp;
use App\Models\Wallet;
use App\Services\StripeHelper;
use BaconQrCode\Renderer\ImageRenderer;
use BaconQrCode\Renderer\Image\SvgImageBackEnd;
use BaconQrCode\Renderer\RendererStyle\RendererStyle;
use BaconQrCode\Writer;
use BaconQrCode\Renderer\RendererStyle\Fill;
use BaconQrCode\Renderer\Color\Rgb;

class DashboardController extends Controller
{
    public function index(Request $request, StripeHelper $stripeHelper)
    {
        // check if user will expire subscription and notify once for each login session
        if (auth()->user()->will_expire === null && auth()->user()->plan_id != null && session('redirected_plan_payment', false) === false) {
            session()->put('redirected_plan_payment', true);
            Toastr::danger(__('Your subscription is not active or renewed yet. Please make a payment to continue using our services.'));
            return inertia_location(route('user.subscription.payment', auth()->user()->plan_id));
        }


        PageHeader::set()
            ->title(__("Dashboard"));
        $user = auth()->user();
        $totalOrders = Order::where('user_id', $user->id)->count();
        $totalCardOrders = CardOrder::where('user_id', $user->id)->count();
        $totalCreditCards = CreditCard::where('user_id', $user->id)->count();
        $totalWallets = Wallet::where('user_id', $user->id)->count();

        $subscriptions = Order::with('plan')
            ->whereUserId($user->id)->latest()->take(5)->get();

        $recentCardOrders = CardOrder::where('user_id', $user->id)->with('user', 'card')
            ->latest()->take(5)->get();
        $recentTopUps = TopUp::where('user_id', $user->id)
            ->with('user:id,first_name,last_name')
            ->latest()->take(5)->get();

        $filterByCard = request('top_up') ?? 'day';
        $dateFormatCard = match ($filterByCard) {
            'year' => "%Y",
            'month' => "%M %Y",
            'week' => "%a",
            default => "%h",
        };
        $topUpSalesOverview = TopUp::query()
            ->where('user_id', $user->id)
            ->where('type', 'qr_top_up')
            ->selectRaw("DATE_FORMAT(top_ups.created_at,'$dateFormatCard') as date, SUM(top_ups.amount) as sales")
            ->when($filterByCard == 'day', function ($query) {
                $query->whereDate('created_at', today())
                    ->groupByRaw('HOUR(top_ups.created_at)');
            })
            ->when($filterByCard == 'week', function ($query) {
                $start = now()->startOfWeek(Carbon::SATURDAY);
                $end = now()->endOfWeek(Carbon::FRIDAY);
                $query->whereBetween('top_ups.created_at', [$start, $end])
                    ->groupByRaw('DAY(top_ups.created_at)');
            })
            ->when($filterByCard == 'month', function ($query) {
                $query->whereYear('created_at', now()->year)
                    ->groupByRaw('MONTH(top_ups.created_at)');
            })
            ->when($filterByCard == 'year', function ($query) {
                $query->groupByRaw('YEAR(top_ups.created_at)');
            })
            ->orderBy('top_ups.created_at', 'asc')
            ->get();
        $cardholder = CardHolder::query()->where('user_id', auth()->id());
        $transactions = [];
        $authorizations = [];
        if ($cardholder->exists()) {
            $authorizations = $stripeHelper->authorizations('user.credit-card.authorizations', $cardholder->first()->cardholder);
            $transactions = $stripeHelper->transactions('user.credit-card.transactions', $cardholder->first()->cardholder);
        }

        $creditCards = CreditCard::query()
            ->where('user_id', auth()->id())
            ->with(['user', 'card:id,card_variant'])
            ->take(5)->get();
        $card_intro_details = get_option('card_intro_details');
        $stripeCurrency = env('STRIPE_CURRENCY', 'usd');

        $qrCodeLink = url('/user/top-ups/' . auth()->user()->uuid . '/payment');
        $qrCode = (new Writer(
            new ImageRenderer(
                new RendererStyle(192, 0, null, null, Fill::uniformColor(
                    new Rgb(255, 255, 255),
                    new Rgb(2, 6, 23)
                )),
                new SvgImageBackEnd
            )
        ))->writeString($qrCodeLink);
        return Inertia::render('User/Dashboard', [
            'totalOrders' => $totalOrders,
            'totalCardOrders' => $totalCardOrders,
            'totalCreditCards' => $totalCreditCards,
            'totalWallets' => $totalWallets,
            'subscriptionsOrders' => $subscriptions,
            'recentCardOrders' => $recentCardOrders,
            'recentTopUps' => $recentTopUps,
            'topUpSalesOverview' => $topUpSalesOverview,
            'transactions' => $transactions['transactions'] ?? [],
            'authorizations' => $authorizations['authorizations'] ?? [],
            'creditCards' => $creditCards,
            'card_intro_details' => $card_intro_details,
            'stripe_currency' => $stripeCurrency,
            'qrCode' => $qrCode,
            'qrCodeLink' => $qrCodeLink,
            'request' => $request
        ]);
    }
}
